import expect, { spyOn } from 'expect';
import ReactDOM from 'react-dom';
import subject from '../src/index';

const HelloRouter = {};

describe('RWRReactRouter', function () {
  afterEach(function () {
    subject.routers = {};
    subject.enabled = false;
  });

  describe('.version', function () {
    it('is present', function () {
      expect(subject.version).toNotEqual(undefined);
    });
  });

  describe('.constructor', function () {
    it('intializes empty components dictionary', function () {
      expect(subject.routers).toEqual({});
    });

    it('by default sets router presence flag to false', function () {
      expect(subject.enabled).toBe(false);
    });
  });

  describe('#register', function () {
    it('adds router to the routers storage', function () {
      subject.register('HelloRouter', HelloRouter);
      expect(subject.routers.HelloRouter).toBe(HelloRouter);
    });
  });

  describe('#get', function () {
    it('returns route by name', function () {
      subject.register('HelloRouter', HelloRouter);
      expect(subject.get('HelloRouter')).toBe(HelloRouter);
    });

    it('returns undefined if route is not found', function () {
      expect(subject.get('HelloRouter')).toBe(undefined);
    });
  });

  describe('#render', function () {
    beforeEach(function () {
      subject.register('HelloRouter', HelloRouter);
    });

    context('when router is present', function () {
      it('throws an error', function () {
        expect(function () {
          subject.render('HelloRouter', { node: 'someNode' });
        })
        .withContext(subject.enabled = true)
        .toThrow(/Error when rendering HelloRouter/);
      });
    });

    context('when router does not exist', function () {
      let ReactDOMSpy;
      const node = { nodeType: 1, nodeName: 'DIV' };
      beforeEach(function () {
        ReactDOMSpy = spyOn(ReactDOM, 'render');
      });

      afterEach(function () {
        expect.restoreSpies();
      });

      it('sets router presence flag to true', function () {
        subject.render('HelloRouter', node);

        expect(subject.enabled).toBe(true);
      });

      it('calls ReactDOM renderer once', function () {
        subject.render('HelloRouter', { nodeType: 1, nodeName: 'DIV' });

        expect(ReactDOMSpy.calls.length).toEqual(1);
        expect(ReactDOMSpy).toHaveBeenCalledWith(HelloRouter, { nodeType: 1, nodeName: 'DIV' });
      });
    });
  });

  describe('#unmount', function () {
    let ReactDOMSpy;
    beforeEach(function () {
      ReactDOMSpy = spyOn(ReactDOM, 'unmountComponentAtNode');
    });

    afterEach(function () {
      expect.restoreSpies();
    });

    it('unmounts router component at specified node', function () {
      subject.unmount({ node: 'someNode' });

      expect(ReactDOMSpy.calls.length).toEqual(1);
      expect(ReactDOMSpy).toHaveBeenCalledWith({ node: 'someNode' });
    });

    it('allows to add new router', function () {
      subject.unmount({ node: 'someNode' });
      expect(subject.enabled).toBe(false);
    });
  });

  describe('#integrationWrapper', function () {
    const node = { nodeType: 1, nodeName: 'DIV' };

    describe('function mount', function () {
      it('calls renderComponent', function () {
        const mountSpy = spyOn(subject, 'render');

        const payload = { name: 'routerName', props: { foo: 'bar' } };
        subject.integrationWrapper.mount(node, payload);

        expect(mountSpy.calls.length).toEqual(1);
        expect(mountSpy).toHaveBeenCalledWith(
          'routerName',
          { nodeType: 1, nodeName: 'DIV' },
          { foo: 'bar' }
        );
      });
    });

    describe('function unmount', function () {
      it('calls unmountComponent', function () {
        const unmountSpy = spyOn(subject, 'unmount');
        subject.integrationWrapper.unmount(node);

        expect(unmountSpy.calls.length).toEqual(1);
        expect(unmountSpy).toHaveBeenCalledWith({ nodeType: 1, nodeName: 'DIV' });
      });
    });
  });
});
